{% extends "base.html" %}

{% block prebody %}
<div class="jumbotron">
    <div class="container">
        <h1>About PandExo </h1>

    </div>
</div>
{% end %}

{% block body %}
<!-- Example row of columns -->
<div class="row">
    <div class="col-md-9">
        <h2>About PandExo</h2>

        <p>PandExo was written using the core capabilities of STScI's Pandeia (Pandeia + Exoplanets = PandExo). The front end of Pandexo
        is designed specifically for exoplanet transit studies. You might ask-- what's the difference between running Pandeia 
        and PandExo? Pandeia requires that you know the number of groups, frames, and integrations. It also only takes in as input a 
        stellar SED. PandExo takes in a stellar SED, a planet spectrum, and the transit duration and provides, as output, the error 
        obtained on your spectrum. You also have the option of optimizing the number of groups, per integration or 
        hard coding your own. PandExo has nice front end plotting functions built in, as well as parallelized bash 
        scripts for running multiple noise models at once. Fundamentally, they also use different methods for computing noise. 
        For a complete explanation of this, see arXiv:1702.01820 </p>
        
        <h2>Links to PandExo Documentation</h2>
        <ul>
        <li><a href="https://natashabatalha.github.io/PandExo/installation.html">Installing PandExo Locally</a></li>
        <li><a href="https://natashabatalha.github.io/PandExo/tutorialjwst.html#editting-input-dictionaries">Editting Input Dict Manually</a></li>
        <li><a href="https://natashabatalha.github.io/PandExo/tutorialjwst.html#analyzing-output">Analyzing PandExo Output</a></li>
        <li><a href="https://github.com/natashabatalha/PandExo/tree/master/notebooks">Notebooks Examples</a></li>
        </ul>

        <h2>Reference Info</h2>
        <p>A detailed description of PandExo is available in Batalha et al. 2016. For specifics about the code please visit the 
        <a href="https://natashabatalha.github.io/PandExo/index.html">PandExo's Github.io</a>. </p>
        
    </div>
</div>
<div class="row">
    <div class="col-md-6">
        <h2>Questions, Comments, Concerns?</h2>
        <p>Have questions/comments/concerns? Check out <a href="https://groups.google.com/d/forum/pandexo">PandExo Google Group</a>
        for a repository of these.</p>
        <p>Found a bug that is NOT discussed in the google group? Email ASAP: <a href="mailto:pandexo@psu.com">pandexo@psu.edu</a></p>
     </div>
</div>            
{% end %}