<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- The above 3 meta tags *must* come first in the head; any other head content must come *after* these tags -->
    <meta name="description" content="">
    <meta name="author" content="">
    <link rel="icon" href="{{ static_url('img/favicon.gif')}}">

    <title>PandExo - ETC for Exoplanets</title>

    <!-- Bootstrap core CSS -->
    <link href="{{ static_url('css/bootstrap.min.css')}}" rel="stylesheet">
    <link href="https://gitcdn.github.io/bootstrap-toggle/2.2.2/css/bootstrap-toggle.min.css" rel="stylesheet">
    <!-- Custom styles for this template -->
    <link href="{{ static_url('css/starter.css')}}" rel="stylesheet">
    <link href="{{ static_url('css/bootstrap-submenu.min.css')}}" rel="stylesheet">
    {% block css %}{% end %}

</head>

<body>

<nav class="navbar navbar-inverse navbar-fixed-top">
    <div class="container">
        <div class="navbar-header">
            <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar"
                    aria-expanded="false" aria-controls="navbar">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a class="navbar-brand" href="https://exoctk.stsci.edu">ExoCTK</a>
        </div>
        <div id="navbar" class="collapse navbar-collapse">
            <ul class="nav navbar-nav">
                <li class="dropdown">
                <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button" aria-haspopup="true" aria-expanded="false">Observation Planning <span class="caret"></span></a>
                 <ul class="dropdown-menu">
                        <li><a href="https://exoctk.stsci.edu/groups_integrations">Groups and Integrations Calculator</a></li>
                        <li><a href="https://exoctk.stsci.edu/contam_visibility">Contamination Overlap</a></li>
                </ul>
                </li>
                <li class="dropdown">
                <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button" aria-haspopup="true" aria-expanded="false">Forward Modeling <span class="caret"></span></a>
                 <ul class="dropdown-menu">
                        <li><a href="https://exoctk.stsci.edu/fortney">Fortney Grid</a></li>
                        <li><a href="https://exoctk.stsci.edu/generic">Generic Grid</a></li>
                </ul>
                </li>
                <li><a href="https://exoctk.stsci.edu/limb_darkening">Limb Darkening Calculator</a></li>
                
                <li class="dropdown">
                <a tabindex="0" href="/" data-toggle="dropdown" role="button" aria-haspopup="true" aria-expanded="false" data-submenu>PandExo <span class="caret"></span></a>
                 <ul class="dropdown-menu">
                        <li><a href="/">Home</a></li>
                        <li class="dropdown-submenu">
                            <a tabindex="0">JWST</a>
                            <ul class="dropdown-menu">
                                <li><a tabindex="0" href="/calculation/new">New Calculation</a></li>
                                <li><a tabindex="0" href="/dashboard">Dashboard</a></li>
                            </ul>
                        </li>
                        <li class="dropdown-submenu">
                            <a tabindex="0">HST</a>
                            <ul class="dropdown-menu">
                                <li><a tabindex="0" href="/calculation/newHST">New Calculation</a></li>
                                <li><a tabindex="0" href="/dashboardhst">Dashboard</a></li>
                            </ul>
                        </li>
                        <li><a href="https://natashabatalha.github.io/PandExo/">DOCS</a></li>
                </ul>
                </li> 
                </li>
                <li><a href="https://exoctk.stsci.edu/zip_data_download">ExoCTK Data Download</a></li>

            </ul>
        </div>
        <!--/.nav-collapse -->
    </div>
</nav>


    {% block prebody %}{% end %}
    <div class="container">

                <p >
                        <img src="{{ static_url('img/ExoCTK_logo.gif')}}" width="450px" />
                </p>
            {% block body %}{% end %}

    </div> <!-- /container -->
    <div class="container">
                <p style="float:right;">    
                        <img src="{{ static_url('img/STScI_logo.png')}}" width="150px" />
                </p>
                
    </div> 


<footer>
    <div class="container">
        <hr></hr>
        <p style="float:right;"><a href="https://doi.org/10.5281/zenodo.2012696"><img src="https://zenodo.org/badge/DOI/10.5281/zenodo.2012696.svg" alt="DOI"></a></p> 
    </div>
</footer>






<script src="//cdnjs.cloudflare.com/ajax/libs/tether/1.4.1/js/tether.min.js"></script>
<script src="http://ajax.googleapis.com/ajax/libs/jquery/1.3/jquery.min.js" type="text/javascript"></script>
<!-- Bootstrap core JavaScript
================================================== -->
<!-- Placed at the end of the document so the pages load faster -->
<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js"></script>
<script src="{{ static_url('js/bootstrap.min.js') }}"></script>
<script src="{{ static_url('js/bootstrap-submenu.min.js') }}"></script>
<script src="{{ static_url('js/docs.js') }}"></script>
<script src="https://gitcdn.github.io/bootstrap-toggle/2.2.2/js/bootstrap-toggle.min.js"></script>
<!--Custom code-->
{% block js %}
{% end %}
</body>
</html>
