@import url('https://fonts.googleapis.com/css?family=Open+Sans:300,400,600');

body {
    font-family: 'Open Sans', "Helvetica Neue", Helvetica, Arial, sans-serif;
    padding-top: 50px;
}

.table > tbody > tr > td {
    vertical-align: middle;
}

.footer {
    /*position: relative;*/
    /*margin-top: -150px; !* negative value of footer height *!*/
    /*height: 150px;*/
    /*clear: both;*/
    /*padding-top: 20px;*/
}

.footer p {
    padding-top: 10px;
    border: 1px solid #ccc;
    border-width: 1px 0 0 0;
}

.container .text-muted {
    margin: 20px 0;
}

/*Add image to jumbotron*/
.jumbotron {
    background: url("../img/bkg-w-logo.jpg") no-repeat center center;
    background-size: cover;
    color: #f5f5f5;
    text-shadow: #0f0f0f;
}

/*Animated loading icon*/
.glyphicon-refresh-animate {
    -animation: spin .7s infinite linear;
    -ms-animation: spin .7s infinite linear;
    -webkit-animation: spinw .7s infinite linear;
    -moz-animation: spinm .7s infinite linear;
}

@keyframes spin {
    from {
        transform: scale(1) rotate(0deg);
    }
    to {
        transform: scale(1) rotate(360deg);
    }
}

@-webkit-keyframes spinw {
    from {
        -webkit-transform: rotate(0deg);
    }
    to {
        -webkit-transform: rotate(360deg);
    }
}

@-moz-keyframes spinm {
    from {
        -moz-transform: rotate(0deg);
    }
    to {
        -moz-transform: rotate(360deg);
    }
}

/* Radio Chooser */
.radio-chooser {
    border: 1px solid #cce6fa;
    border-radius: 4px;
    box-sizing: border-box;
    display: flex;
    font-family: 'Open Sans', sans-serif;
    margin: 15px 0;
}
.radio-chooser-item {
    width: 50%;
    box-sizing: border-box;
    padding: 20px;
    display: inline-block;
    background: white;
}
.radio-chooser-item.radio-chooser-selected {
    background: #f4faff;
}
.radio-chooser-item:first-child {
    border-right: 1px solid #cce6fa;
    box-sizing: border-box;
}
.radio-chooser-item:hover {
    background: #f4faff;
}
.radio-chooser-content {
    cursor: pointer;
    width: 100%;
    margin: 0;
    padding: 5px 15px;
}
.radio-chooser-content input {
    margin: 0 15px 0 0 !important;
    line-height: 20px;
    vertical-align: middle;
}
.radio-chooser-title {
    max-width: 270px;
    display: inline-block;
    vertical-align: middle;
    font-weight: 100;
    font-size: 18px;
}